/* IMSpector - Instant Messenger Transparent Proxy Service
 * http://www.imspector.org/
 * (c) Lawrence Manning <lawrence@aslak.net>, 2006
 *          
 * Released under the GPL v2, with the OpenSSL exception described
 * in the README file. */

/*!
 * \file filterplugin.h
 *
 * \brief FilterPlugin class.
 */

/*!
 * \brief Info structure for a filter plugin. Nice and trivial.
 */
struct filterplugininfo
{
	std::string pluginname; //!< The plugin's all have names.
};

typedef bool (*initfilterplugintype)(struct filterplugininfo &filterplugininfo,
	class Options &options, bool debugmode);
typedef void (*closefilterplugintype)(void);
typedef bool (*filtertype)(char *originalbuffer, char *modifiedbuffer, struct imevent &imevent);

/*!
 * \brief A FilterPlugin class encapsulates a filter plugin shared object file.
 *
 * Filter plugins, as per all other plugin types, are two things: an object
 * of the class FilterPlugin, and a shared object wrappered by the object. These
 * objects are usually stored on a vector, so that (say) all the filter plugins 
 * can be applied to an event.
 *
 * All plugin types have a plugininfo struct, that describes the plugin. For
 * filter plugins, this is just a name.
 */
class FilterPlugin
{
	public:
		/*! \brief This is public so we don't have to mess with accessors. */
		struct filterplugininfo filterplugininfo;
		
		FilterPlugin();
		~FilterPlugin();
		bool loadplugin(std::string filename);
		bool unloadplugin(void);
		
		bool callinitfilterplugin(class Options &options, bool debugmode);
		void callclosefilterplugin(void);
		bool callfilter(char *originalbuffer, char *modifiedbuffer, struct imevent &imevent);

	private:
		void *handle; //!< Our dlopen handle.

		initfilterplugintype initfilterplugin;   //!< The initplugin funct pointer.
		closefilterplugintype closefilterplugin; //!< The closeplugin funct pointer.
		filtertype filter;                       //!< The filter funct pointer.
};

