/* IMSpector - Instant Messenger Transparent Proxy Service
 * http://www.imspector.org/
 * (c) Lawrence Manning <lawrence@aslak.net>, 2006
 * 
 * Contributions from:
 *     Ryan Wagoner <ryan@wgnrs.dynu.com>, 2006
 *          
 * Released under the GPL v2, with the OpenSSL exception described
 * in the README file. */

/*!
 * \file imspector.h
 *
 * \brief Main include file.
 *
 * Instead of having seperate include files, must .cpp files just include
 * this one.
 */

#include <stdlib.h>
#include <stdio.h>
#include <dlfcn.h>
#include <fcntl.h>
#include <glob.h>
#include <syslog.h>
#include <pwd.h>
#include <grp.h> 
#include <unistd.h>
#include <signal.h>
#include <stdarg.h>
#include <netdb.h>
#include <limits.h>
#include <iconv.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/un.h>
#include <sys/errno.h>
#include <sys/stat.h>
#include <sys/ioctl.h>

#include <net/if.h>

#include <netinet/in.h>

#include <arpa/inet.h>

#ifdef HAVE_SSL
#include <openssl/ssl.h>
#include <openssl/bio.h>
#include <openssl/err.h>
#endif

#if defined(__linux__)
#include <linux/types.h>
#include <linux/netfilter_ipv4.h>
#define LINUX_NETFILTER 1
#endif

#if defined(__FreeBSD__) || defined(__OpenBSD__)
#include <net/pfvar.h>
#define PF_TRANSPARENT 1
#endif

#include <algorithm>
#include <string>
#include <vector>
#include <map>
#include <fstream>
#include <iostream>

#define STRING_SIZE 1024  //!< The length of a typical string.
#define BUFFER_SIZE 65536 //!< The length of a typical buffer.


/*! Where packet trace files go.
 * \note This really should be configurable. */
#define TRACE_DIR "/tmp/trace"

#ifndef UNIX_PATH_MAX
/*! Some systems do not seem to have this defined. */
#define UNIX_PATH_MAX 108
#endif

#define TYPE_NULL 0   //!< No event type set yet.
#define TYPE_MSG 1    //!< A message event.
#define TYPE_FILE 2   //!< A file transfer.
#define TYPE_TYPING 3 //!< User is typing.
#define TYPE_WEBCAM 4 //!< User wants to start a webcam session.

/*!
 * \brief Describes an offset and length into a buffer of data.
 *
 * This is typically used when wanting to censor a part of an IM packet that
 * contains the acutal message text.
 */
struct messageextent
{
	int start;  //!< Number of bytes into buffer to start from.
	int length; //!< Number of bytes to count.
};

/*!
 * \brief Struct represents an Instant Messaging event.
 *
 * This struct is used extensivly by IMSpector. A vector of these is used
 * to queue up events for logging, and they also hold "derived" data
 * including a flag for wether the message filtered by a filtering plugin.
 */
struct imevent
{
	time_t timestamp;          //!< UNIX timestamp.
	std::string clientaddress; //!< IP:Port
	std::string protocolname;  //!< From the protocol plugin.
	bool outgoing;             //!< Event is local->remote.
	int type;                  //!< One of TYPE_
	std::string localid;       //!< Local party.
	std::string remoteid;      //!< Remote party.
	bool filtered;             //!< Filter plugin blocked event.
	std::string categories;	   //!< Filter plugins append here.
	std::string eventdata;     //!< Usually message text.
	/*! \brief Not used by logging plugins. */
	struct messageextent messageextent;
};

/*!
 * \brief Holds a message to generate.
 *
 * Messages are generated as responses in a protocol-agnostic way, so the
 * message text should be in plain-text.
 */
struct response
{
	bool outgoing;    //!< true for outgoing.
	std::string text; //!< The message text to send.
};

#include "socket.h"
#include "options.h"
#include "tools.h"
#include "sslstate.h"
#include "protocolplugin.h"
#include "loggingplugin.h"
#include "filterplugin.h"
#include "responderplugin.h"
