/* IMSpector - Instant Messenger Transparent Proxy Service
 * http://www.imspector.org/
 * (c) Lawrence Manning <lawrence@aslak.net>, 2006
 *          
 * Released under the GPL v2, with the OpenSSL exception described
 * in the README file. */

/*!
 * \file loggingplugin.h
 *
 * \brief LoggingPlugin class.
 */

/*!
 * \brief Info structure for a logging plugin. Nice and trivial.
 */
struct loggingplugininfo
{
	std::string pluginname;
};

typedef bool (*initloggingplugintype)(struct loggingplugininfo &loggingplugininfo,
	class Options &options, bool debugmode);
typedef void (*closeloggingplugintype)(void);
typedef int (*logeventstype)(std::vector<struct imevent> &imevents);

/*!
 * \brief A LoggingPlugin class encapsulates a logging plugin shared object file.
 *
 * Logging plugins, as per all other plugin types, are two things: an object
 * of the class LoggingPlugin, and a shared object wrappered by the object. These
 * objects are usually stored on a vector, so that (say) all the logging plugins 
 * can be applied to an event to log in the various ways available.
 *
 * All plugin types have a plugininfo struct, that describes the plugin. For
 * filter plugins, this is just a name.
 */
class LoggingPlugin
{
	public:
		/*! \brief This is public so we don't have to mess with accessors. */
		struct loggingplugininfo loggingplugininfo;
	
		LoggingPlugin();
		~LoggingPlugin();
		bool loadplugin(std::string filename);
		bool unloadplugin(void);
		
		bool callinitloggingplugin(class Options &options, bool debugmode);
		void callcloseloggingplugin(void);
		int calllogevents(std::vector<struct imevent> &imevents);

	private:
		void *handle; //!< Our dlopen handle.

		initloggingplugintype initloggingplugin;   //!< The initplugin funct handle.
		closeloggingplugintype closeloggingplugin; //!< The closeplugin funct handle.
		logeventstype logevents;                   //!< The logevents funct handle.
};
