/* IMSpector - Instant Messenger Transparent Proxy Service
 * http://www.imspector.org/
 * (c) Lawrence Manning <lawrence@aslak.net>, 2006
 *          
 * Released under the GPL v2, with the OpenSSL exception described
 * in the README file. */

/*!
 * \file options.cpp
 *
 * \brief Options class defintion.
 */

#include "imspector.h"

/*!
 * \brief Reads a file of options into the options object.
 *
 * Settings, consisting of a key and a value, are added to the map, so it is
 * possible to call this more then once to overlay options into the same
 * object.
 *
 * \param filename The filename to read
 * \return true for success
 */
bool Options::readoptionsfile(std::string filename)
{
	FILE *hfile = NULL;
	char buffer[STRING_SIZE];
	char *equals;
	
	memset(buffer, 0, STRING_SIZE);
	
	if (!(hfile = fopen(filename.c_str(), "r")))
		return false;
		
	while (fgets(buffer, STRING_SIZE, hfile))
	{
		/* Remove the \n. */
		char *t = strchr(buffer, '\n');
		if (t) *t = '\0';		
        
		/* Ignore lines with no equal signs or comment lines. */
		if (buffer[0] == '#') continue;
		if (!(equals = strchr(buffer, '='))) continue;

		*equals = '\0';
		
		params[buffer] = equals + 1;
	}
	
	fclose(hfile);
	
	return true;
}

/*!
 * \brief Overloaded [] operator to read a key
 */
std::string Options::operator[](const char *key)
{
	return params[key];
}

/*!
 * \brief Returns a vector of all the available option keys.
 *
 * \returns A list of all the keys as a vector of strings
 * \note A key can reference an empty string
 */
std::vector<std::string> Options::getkeys(void)
{
	std::vector<std::string> keys;
	
	for (std::map<std::string, std::string>::iterator i = params.begin();
		i != params.end(); i++)
	{
		keys.push_back((*i).first);
	}
	
	return keys;
}
