/* IMSpector - Instant Messenger Transparent Proxy Service
 * http://www.imspector.org/
 * (c) Lawrence Manning <lawrence@aslak.net>, 2006
 *          
 * Released under the GPL v2, with the OpenSSL exception described
 * in the README file. */

/*!
 * \file options.h
 *
 * \brief Options class.
 */

/*!
 *
 * \class Options
 *
 * \brief Settings reader and container
 *
 * Contains a function from reading options from a file, and a map holding
 * the actual options.  For ease of use, there is also a method for obtaining
 * all the option keys in a vector. 
 *
 * A Settings file has lines like this:
 * 
 * key1=value
 *
 * Comments are also supported; a hash at the start of a line makes the
 * parser ignore the rest of the line.
 *
 * \see imspector.conf
 */

class Options
{
	public:
		bool readoptionsfile(std::string filename);
		std::string operator[](const char *key);
		std::vector<std::string> getkeys(void);
	
	private:
		/*! \brief The map of options. */
		std::map<std::string, std::string> params;
};
