/* IMSpector - Instant Messenger Transparent Proxy Service
 * http://www.imspector.org/
 * (c) Lawrence Manning <lawrence@aslak.net>, 2006
 *          
 * Released under the GPL v2, with the OpenSSL exception described
 * in the README file. */

/*!
 * \file responderplugin.h
 *
 * \brief ResponderPlugin class.
 */

/*!
 * \brief Info structure for a responder plugin. Nice and trivial.
 */
struct responderplugininfo
{
	std::string pluginname;
};

typedef bool (*initresponderplugintype)(struct responderplugininfo &responderplugininfo,
	class Options &options, bool debugmode);
typedef void (*closeresponderplugintype)(void);
typedef int (*generateresponsestype)(std::vector<struct imevent> &imevents,
	std::vector<struct response> &responses);

/*!
 * \brief A ResponderPlugin class encapsulates a filter plugin shared object file.
 *
 * Responder plugins, as per all other plugin types, are two things: an object
 * of the class ResponderPlugin, and a shared object wrappered by the object. These
 * objects are usually stored on a vector, so that (say) all the responder plugins 
 * can be applied to an event and the responses generated.
 *
 * All plugin types have a plugininfo struct, that describes the plugin. For
 * filter plugins, this is just a name.
 */
class ResponderPlugin
{
	public:
		/*! \brief This is public so we don't have to mess with accessors. */
		struct responderplugininfo responderplugininfo;
	
		ResponderPlugin();
		~ResponderPlugin();
		bool loadplugin(std::string filename);
		bool unloadplugin(void);
		
		bool callinitresponderplugin(class Options &options, bool debugmode);
		void callcloseresponderplugin(void);
		int callgenerateresponses(std::vector<struct imevent> &imevents,
			std::vector<struct response> &responses);

	private:
		void *handle; //!< Our dlopen handle.

		initresponderplugintype initresponderplugin;   //!< The initplugin funct pointer.
		closeresponderplugintype closeresponderplugin; //!< The closeplugin funct pointer.
		generateresponsestype generateresponses;       //!< The genrateresponses funct pointer.
};
