/* IMSpector - Instant Messenger Transparent Proxy Service
 * http://www.imspector.org/
 * (c) Lawrence Manning <lawrence@aslak.net>, 2006
 * 
 * Contributions from:
 *     Ryan Wagoner <ryan@wgnrs.dynu.com>, 2006
 *     Simon Brassington <simon@the-brassingtons.co.uk>, 2007
 *          
 * Released under the GPL v2, with the OpenSSL exception described
 * in the README file. */

/*!
 * \file socket.h
 *
 * \brief Socket class.
 */
 
/*!
 * \brief Wrapper for a sockaddr
 *
 * This is used so we can use one inernal structure for UNIX sockets and
 * normal network sockets.  It's abit weird.
 */
struct mysockaddr
{
#if defined (__linux__)
	__SOCKADDR_COMMON (sa_);
#else
	unsigned char   sa_len;
	sa_family_t  sa_family;
#endif
	char sa_data[STRING_SIZE];
};

/*!
 * \brief A wrapper for a socket fd.
 *
 * This is a general purpose socket fd wrapper with a few fancy features,
 * such as supporting SSL mode, and it works equally well with UNIX sockets
 * as TCP/UDP.
 *
 * It also wraps the gory details of socket addresses, by using IP:Port
 * format addresses as strings in it's API in place of the usual struct
 * sockaddr. Currently only unbuffered I/O is supported, with some cheesy
 * functions for reading up to a newline in char-by-char fashion.
 *
 * There is no need to subclass this class, just add to it if you need more
 * functionality.
 */
class Socket
{
	public:
		Socket(int domainin, int typein);
		~Socket();
		bool listensocket(std::string localaddress);
		bool awaitconnection(class Socket &clientsocket, std::string &clientaddress);
		std::string getredirectaddress(void);
		bool connectsocket(std::string remoteaddress, std::string interface);

#ifdef HAVE_SSL
		bool enablessl(SSL_CTX *ctx);
		bool sslaccept(void);
		bool sslconnect(void);
		std::string getpeercommonname(void);
		int getvalidatecertresult(void);
#endif
		int senddata(const char *buffer, int length);
		bool sendalldata(const char *buffer, int length);
		int sendline(const char *string, int length);
		int recvdata(char *buffer, int length);
		bool recvalldata(char *buffer, int length);
		int recvline(char *string, int length);
		int getfd(void);
		void closesocket(void);
		
	private:
		void setfd(int fdin);
		struct mysockaddr stringtosockaddr(std::string address);
		std::string sockaddrtostring(struct mysockaddr *pmyname);
	
		int domain;     //!< Constructor parameter.
		int type;       //!< Constructor parameter.
		int fd;         //!< The underlying File Descriptor.
		
#ifdef HAVE_SSL
		SSL *ssl;       //!< Top level SSL data structure.
		X509 *peercert; //!< The server server on connected sockets.
#endif
};
