/* IMSpector - Instant Messenger Transparent Proxy Service
 * http://www.imspector.org/
 * (c) Lawrence Manning <lawrence@aslak.net>, 2006
 *          
 * Released under the GPL v2, with the OpenSSL exception described
 * in the README file. */

/*!
 * \file sslstate.h
 *
 * \brief SSLState class.
 */

/*!
 * \brief SSLState class encapsulates all our SSL stuff.
 *
 * Container for SSL state information, cert store paths, and other handy
 * bits.
 *
 * SSL support is provided by a mixture of cert-related stuff, in the SSLState
 * class, and Socket-level things provided by the Socket class. Cert creatiion
 * and signing is handled by a forked off process created by this class.
 *
 * SSL switching, both at initial connection time, and when a ProtocolPlugin 
 * requests it, is done in doproxy().
 *
 * If HAVE_SSL isnt defined, this is an empty class. Other preprocessor checks
 * will prevent any function being called. Hiding the functionality in this way
 * seemed like the easier way to do it.
 */
class SSLState
{
#ifdef HAVE_SSL
	public:
		SSLState(void);
		bool init(class Options &options, bool debugmode);
		void free(void);
		bool imserversocktossl(class Socket &imserversock);
		bool clientsocktossl(class Socket &clientsock);
		
	private:
		bool sslcertclient(std::string commandline, bool selfsigned);
		bool loadcert(SSL_CTX *ctx, std::string domain, std::string certfilename);

		bool sslcertserver(std::string sslcertdir);
		bool signcert(std::string commonname, bool selfsigned);
		
		std::string formatcertfilename(std::string commonname);
		
		std::string sslcertdir;   //!< Where to put our MITM certs.
		std::string sslverifydir; //!< Where to find the CAs to validate against.
		int sslverify;            //!< Our VERIFY_ choice.
		
		bool localdebugmode;      //!< Saves us from passing this around.
		SSL_METHOD *method;       //!< openssl method handle.
		SSL_CTX *connectctx;      //!< Our IM server-sde CTX.
		SSL_CTX *ctx;             //!< Our IM client-side CTX.

		X509 *cacert;             //!< The CA cert.
		EVP_PKEY *cakey;          //!< And it's key.
		EVP_PKEY *serverkey;      //!< The private server key.
		
#endif
};
